Package: glow
Title: Make Plots that Glow
Version: 0.13.0
Date: 2024-09-24
Authors@R: c(
    person("Travers", "Ching", email = "traversc@gmail.com", role = c("aut", "cre", "cph"))
	)
Maintainer: Travers Ching <traversc@gmail.com>
Description: Provides a framework for creating plots with glowing points.
License: GPL-3
Biarch: true
Encoding: UTF-8
LinkingTo: Rcpp, RcppEigen, RcppParallel
Imports: Rcpp, RcppParallel (>= 5.1.2), R6
Depends: ggplot2
Suggests: knitr, rmarkdown, viridisLite, magick, EBImage, qs2
VignetteBuilder: knitr
RoxygenNote: 7.3.2
URL: https://github.com/traversc/glow
BugReports: https://github.com/traversc/glow/issues
NeedsCompilation: yes
Packaged: 2024-09-25 05:11:08 UTC; tching
Author: Travers Ching [aut, cre, cph]
Repository: CRAN
Date/Publication: 2024-09-25 12:30:06 UTC
Built: R 4.6.0; x86_64-apple-darwin20; 2026-04-28 04:24:07 UTC; unix
Archs: glow.so.dSYM
